// lftvideodialog.cpp
// LFTVideo.cpp  v1.71
//               14.5.2003
// 
// fr weitere Informationen:
// siehe header von main.cpp

#include "lftvideodialog.h"

#include <qframe.h>
#include <qpushbutton.h>
#include <qlcdnumber.h>
#include <qslider.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>
#include <qmessagebox.h>
#include <math.h>			// for floor()

LFTVideoDialog::LFTVideoDialog( QWidget* parent, 
							    const char* name, bool modal, WFlags f )
	: LFTVideoDialogBase( parent, name, modal, f )
{
	this->setProperty( "caption", tr( "v1.76 LFTVideo" ) );

	KonfigDialog=new SitzungsKonfigurationDialog ( this, "KonfigDialog", TRUE,
		Qt::WStyle_Customize | Qt::WStyle_Title  | Qt::WStyle_SysMenu);

	QFont *dialog_font=new QFont (  KonfigDialog->font() );
	dialog_font->setFamily( "Arial" ); 
	dialog_font->setPointSize( 8 );
	KonfigDialog->setFont ( *dialog_font, TRUE) ;
	BtTherapeut_NLF->setText("nicht\nLs.Sprache (6)");
	BtTherapeut_NLF->setProperty( "accel", 268435510 );
	BtKlient_NLF->setText("nicht\nLs.Sprache (3)");
	BtKlient_NLF->setProperty( "accel", 268435507 );

	// signals and slots connections for buttons
	connect( (const QObject*) BtKonfiguration, SIGNAL(clicked() ),
		this, SLOT(openKonfigDialog() ) );
	connect( (const QObject*) BtRun, SIGNAL(toggled(bool) ),
		this, SLOT(RatingBeginnOderPause(bool) ) );
	connect ((const QObject*) BtGrpRating, SIGNAL(pressed(int)),
		this, SLOT(SpeichereRatingButton(int)) );
	connect ((const QObject*) BtSave, SIGNAL(pressed()),
		this, SLOT(RatingEnde()) );
	connect ((const QObject*) BtKorrigieren, SIGNAL(toggled(bool)),
		this, SLOT(KorrigierenSchalter(bool)) );
	connect ((const QObject*) PushButtonCountdown, SIGNAL(toggled(bool)),
		this, SLOT(CountdownStart(bool)) );
	// Countdown Timer:
	CountdownTimer=new QTimer();
	connect( CountdownTimer, SIGNAL(timeout()),
             this, SLOT(Countdown()) );

	// der LCD update Timer:
	zaehlwerk=new QTime();
	SekundenTimer=new QTimer();
	connect( SekundenTimer, SIGNAL(timeout()),
             this, SLOT(aktualisiereZeitanzeige()) );
	connect( SldSecs, SIGNAL(valueChanged(int)),
		this, SLOT(SliderCheckFaerbung()) );
	connect( SldMins, SIGNAL(valueChanged(int)),
		this, SLOT(SliderCheckFaerbung()) );

	// Neubeginn Speicher und GUI:
	initNullSetzungen(); // NEW RatingVerlauf, weitergefhrt=false, 
						 // Anzahl->0, Dauer->0, letzteEinsch->0
	                     // Sld->0, Run->disabled, Counter->disabled,
						 // KonfigOK->disabled, KonfigKlient->""
};

// helper
QString giveDateStamp (){
	QDate d=QDate::currentDate();
	QString ts="%1%2%3"; 
	ts=ts.arg(d.year())
		 .arg(leadingZero(d.month()))
		 .arg(leadingZero(d.day()));
	return ts;
};

QString giveTimeStamp (){
	QTime t=QTime::currentTime();
	QString ts="%1%2%3"; 
	ts=ts.arg(leadingZero(t.hour()))
		 .arg(leadingZero(t.minute()))
		 .arg(leadingZero(t.second()));
	return ts;
};

void LFTVideoDialog::initNullSetzungen(){

	Verlauf=new RatingVerlauf();
	weitergefuehrteSession=false;
	PauseGedrueckt=false;
	TextLabelAnzahl->setText("0");
	TextLabelMittlereDauer->setText("00:00");
	letzterEinschaetzungsZeitpunkt=MinSec(0,0);
	// resette Zaehlwerk
	SldSecs->setValue(0);
	SldMins->setValue(0);
	// zuerst muss immer Konfiguration ausgefllt werden, sonst geht gar nix:
	FrmRun->setEnabled(false);
	FrmCounter->setEnabled(false);
	KonfigDialog->buttonOk->setEnabled(false);
	KonfigDialog->LineEditKlient->setText("");
}

void LFTVideoDialog::CountdownStart(bool start){
	if (start){
		CountdownTimer->stop();
		CountdownTimer->start(500);
		BtRun->setText("10");
		infoline1->setText("Wichtig ist eine SEHR genaue Synchronisierung "
			               "mit dem Videocounter! Bitte im Zweifel mehrmals "
						   "versuchen...");
	}
}

void LFTVideoDialog::Countdown(){
	int i=BtRun->text().toInt();
	i--;
	if (i<0) {
		CountdownTimer->stop();
		BtRun->setText("&Run");
		infoline1->setText("");
		this->setEnabled(true);
		BtRun->toggle();
		PushButtonCountdown->toggle();
	}else{
		QString temp="%1";
		temp=temp.arg(i);
		BtRun->setText(temp);
	}
}


void LFTVideoDialog::SliderCheckFaerbung (){
	MinSec slider(SldMins->value(), SldSecs->value());

	if (   ! (letzterEinschaetzungsZeitpunkt < slider) 
		&& ! (letzterEinschaetzungsZeitpunkt==MinSec(0,0)))
	{
		LCDSecs->setPalette( QPalette( QColor(255,70,70) ) );
		LCDMins->setPalette( QPalette( QColor(255,70,70) ) );
		infoline2->setText("dieser Zeit-Bereich wurde schon eingeschtzt!!!"
			"   Bitte nicht nochmal einschtzen, das fhrt garantiert zu "
			"Chaos...");
	}else{
		if (LCDSecs->palette() == QPalette( QColor(255,70,70) ) ){
			LCDSecs->unsetPalette( );
			LCDMins->unsetPalette( );
			infoline2->setText("");
		}
	}
}

void LFTVideoDialog::KorrigierenSchalter(bool toggled){
	if(toggled){
		//qDebug("on");
		FrmKorrigieren->setPalette( QPalette( QColor(255,70,70) ) );
		infoline2->setPalette( QPalette( QColor(255,70,70) ) );

		falscherButton=Verlauf->RatingAuslesen(Verlauf->BisherLetzterMoment())
			.Einschaetzung;
		if (falscherButton){
			BtGrpRating->find(falscherButton)->
				              setPalette( QPalette( QColor(255,150,150) ) );
			infoline2->setText("Bitte nun die letzte Entscheidung korrigieren");
		} else infoline2->setText("nix zu korrigieren...");
	}else{
		//qDebug("off");
		FrmKorrigieren->unsetPalette( );
		infoline2->unsetPalette( );
		if (falscherButton>0)
			BtGrpRating->find(falscherButton)->unsetPalette(  );
		BtKorrigieren->setOn(false);
		infoline2->setText("");
	}
}

void LFTVideoDialog::SpeichereRatingButton(int BtNumber){
	if (BtKorrigieren->isOn()){
		//qDebug("Korrektur");
		MinSec letzterEintrag=Verlauf->BisherLetzterMoment();
		if (letzterEintrag==MinSec(0,0))
			infoline2->setText("Noch nix zu korrigieren.");
		else 
		{
			Verlauf->RatingLoeschen (letzterEintrag);
			Verlauf->RatingAnhaengen(letzterEintrag,(RatingSkala)BtNumber);
			infoline2->setText("Die letzte Einschaetzung wurde korrigiert.");
		}
		KorrigierenSchalter(false);

	}else
	{
		MinSec Zeitpunkt=MinSec(vergangeneMinuten(), vergangeneSekunden());
		Verlauf->RatingAnhaengen(Zeitpunkt, (RatingSkala) BtNumber);
		// qDebug(Zeitpunkt.toQString()+" %d", BtNumber);

		// Anzahl-Anzeige
		int anz=TextLabelAnzahl->text().toInt() + 1 ;
		QString temp="%1";
		temp=temp.arg(anz);
		TextLabelAnzahl->setText(temp);
		// Durchschnittsdauer-Anzeige
		double dauer= Zeitpunkt.toSecs() / (double) anz;
		temp=MinSec(0, dauer).toQString();
		TextLabelMittlereDauer->setText(temp);
	}
}

void LFTVideoDialog::RatingEnde(){
	MinSec Zeitpunkt=MinSec(vergangeneMinuten(), vergangeneSekunden());
	// qDebug("Gesamter Verlauf bis "+Zeitpunkt.toQString()+" :");
	// qDebug(Verlauf->printGesamtenRatingVerlauf(MinSec(0,0), Zeitpunkt));

	// SPEICHERE Verlauf 
	VerlaufSpeichern *Speicherer=new VerlaufSpeichern (this);
	Speicherer->setzePointerAufDaten(Verlauf);
	QString filenameVerlauf= Speicherer->gibFilename(
		KonfigDialog->LineEditKlient->text(),
		KonfigDialog->LineEditRater->text());
	bool erfolgreichVerlauf;
	erfolgreichVerlauf = Speicherer->
		                   speichereDaten(filenameVerlauf,"Rating Verlauf");
	delete Speicherer;
    qDebug("Speichern war erfolgreich: %d",erfolgreichVerlauf);

	// SPEICHERE Info
	InfoSpeichern *InfoSpeicherer=new InfoSpeichern (this);
	InfoSpeicherer->merkDirDaten(TextLabelAnzahl->text(), 
								 TextLabelMittlereDauer->text(),
								 Verlauf->BisherLetzterMoment(),
								 weitergefuehrteSession,
								 PauseGedrueckt);
	QString filenameInfo= InfoSpeicherer->gibFilename(
		  KonfigDialog->LineEditKlient->text(),
		  KonfigDialog->LineEditRater->text());
	bool erfolgreichInfo;
	erfolgreichInfo = InfoSpeicherer->
		                    speichereDaten(filenameInfo,"Rating Verlauf");
	delete InfoSpeicherer;
    qDebug("InfoSpeichern war erfolgreich: %d",erfolgreichInfo);

	if (erfolgreichVerlauf && erfolgreichInfo){
		infoline1->setText("Verlauf und Info gespeichert: "
		+filenameVerlauf+" und "+filenameInfo);
		infoline2->setText("Alle flchtigen Daten gelscht,\n"
			               "Neubeginn mglich.");
	}else {
		infoline1->setText("Irgendwelche Speicherprobleme\n"
		"Am besten nochmal versuchen...");
		return;
	}
	
	// TODO: Auswerten

	// zerstre Daten:
	Verlauf->Sinneinheiten.clear();
	delete Verlauf;

	// Neubeginn Speicher und GUI:
	initNullSetzungen(); // NEW RatingVerlauf, weitergefhrt=false, 
						 // Anzahl->0, Dauer->0, letzteEinsch->0
	                     // Sld->0, Run->disabled, Counter->disabled,
						 // KonfigOK->disabled, KonfigKlient->""

}


double LFTVideoDialog::vergangeneZeitInSekunden(){
	double secs = StartLCDSekunden+StartLCDMinuten*60;
	// vergangene MILLIsekunden sind nur im laufenden Timer gespeichert
	if (SekundenTimer->isActive()) 
		secs += ((double) zaehlwerk->elapsed()) / (double) 1000;
	return secs;
}


double LFTVideoDialog::vergangeneSekunden(){
	double secs=vergangeneZeitInSekunden();
	int nurSecs=((int) secs) % 60;
	return nurSecs + nachkomma(secs);
}

int LFTVideoDialog::vergangeneMinuten(){
	int mins=vergangeneZeitInSekunden()/60;
	return mins;
}

void LFTVideoDialog::aktualisiereZeitanzeige(){
	int secs=(int) vergangeneZeitInSekunden();
	SldSecs->setValue(secs%60);
	int mins=vergangeneMinuten();
	SldMins->setValue(mins);
};


void LFTVideoDialog::RatingBeginnOderPause(bool beginn){

	if(beginn){
		// TODO / EVTL: Sicherste Variante wre LSCHEN der Restdaten!
		// Wenn restart, dann checken, ob schon gerateter Bereich 
		// betroffen, (dann Abfrage und bei OK:)
		// den ganzen Rest lschen (die Folge wre Neu-Rating!)
		// bei Cancel: RatingBeginnOderPause(false)

		infoline1->setText("Abschalten des Run-Buttons "
			"PAUSIERT das Zhlwerk (nur benutzen wenn NICHT die "
			"Inter-Rater-bereinstimmung gemessen werden muss!)");
		infoline2->setText("Das Zhlwerk luft. Am Ende jeder Sinneinheit "
			               "eine der 7 Tasten drcken.");

		StartLCDSekunden=LCDSecs->value();
		StartLCDMinuten=LCDMins->value();
		SldMins->setEnabled(false);
		SldSecs->setEnabled(false);
		if(MinSec(0,0) < Verlauf->BisherLetzterMoment() ){
			qDebug("pause gedrckt"); // taucht spter in _info.dat auf
			PauseGedrueckt=true;	  // macht nmlich inter-Rater-Vergleich schwerer!
		}
		zaehlwerk->restart();
		SekundenTimer->stop();
		SekundenTimer->start(1000);
	} else
	{
		infoline1->setText("Zhlwerk angehalten (PAUSE)");
		infoline2->setText("");
		SekundenTimer->stop();
		KorrigierenSchalter(false);
		letzterEinschaetzungsZeitpunkt=Verlauf->BisherLetzterMoment();
		SldMins->setEnabled(true);
		SldSecs->setEnabled(true);
	}
}

void LFTVideoDialog::openKonfigDialog(){

	// TODO / EVTL: 
	// Im Falle von Cancel die alten Eintrge wiederherstellen?

	KonfigDialog->LineEditDatum->setText(QDate::currentDate().toString());
	KonfigDialog->LineEditUhrzeit->setText(QTime::currentTime().toString());
	KonfigDialog->show();

	// wenn's geht mit alten Daten weitermachen 
	if (   KonfigDialog->ja->isChecked() 
		&& KonfigDialog->TextLabelExistiert->text()=="schon")
	{
		VerlaufSpeichern *Speicherer=new VerlaufSpeichern (this);
		Speicherer->setzePointerAufDaten(Verlauf);
		QString filename= Speicherer->gibFilename(
			KonfigDialog->LineEditKlient->text(),
			KonfigDialog->LineEditRater->text());
		bool erfolgreich = Speicherer->leseDaten(filename,"Rating Verlauf");
		if (erfolgreich) {
			weitergefuehrteSession=true; 

			if (MinSec(0,0) < Verlauf->BisherLetzterMoment()) // schon Daten da?
				PauseGedrueckt=true;  // TODO: Die ist vereinfachend.
			                          // Besser wre dies aus _Info auszulesen
			                          // Denn nur einlesen und sofort wieder speichern
			                          // macht nix kaputt an Zeitkohrenz...
		}
		delete Speicherer;
		QString infotext="Daten wurden";
		infotext+=(erfolgreich?" ":" NICHT ");
		infotext+="erfolgreich aus ";
		infotext+=filename;
		infotext+=" eingelesen";
		infoline1->setText(infotext);

		letzterEinschaetzungsZeitpunkt=Verlauf->BisherLetzterMoment();
		SldMins->setValue(letzterEinschaetzungsZeitpunkt.mins());
		SldSecs->setValue(letzterEinschaetzungsZeitpunkt.secs());

		// Anzahl-Anzeige
		int anz=Verlauf->Sinneinheiten.size()-1 ; // -1 wegen Nullpunkt
		QString temp="%1";
		temp=temp.arg(anz);
		TextLabelAnzahl->setText(temp);
		// Durchschnittsdauer-Anzeige
		double dauer= letzterEinschaetzungsZeitpunkt.toSecs() / (double) anz;
		temp=MinSec(0, dauer).toQString();
		TextLabelMittlereDauer->setText(temp);
	}

	if (KonfigDialog->buttonOk->isEnabled()){
		infoline1->setText("OK. Es kann losgehen...");
		FrmRun->setEnabled(true);
		FrmCounter->setEnabled(true);
	}else	{
		infoline1->setText("Zuerst die Sitzung konfigurieren");
		FrmRun->setEnabled(false);
		FrmCounter->setEnabled(false);
	}
};


LFTVideoDialog::~LFTVideoDialog()
// ein umfangreicher Destruktor, der erst mit dem Nutzer schimpft
// und dann versucht, die Daten unter einem Timestamp-Namen abzuspeichern.
// 
{
	int verloren=Verlauf->Sinneinheiten.size();
	MinSec arbeitszeit=Verlauf->BisherLetzterMoment();

	if (verloren>1)
	{
		QString dasDatum="SIKO_"; dasDatum+=giveDateStamp();
		QString dieZeit=giveTimeStamp();
		QString filename1=(dasDatum + "_" + dieZeit +"_info.dat").lower();
		QString filename2=(dasDatum + "_" + dieZeit +"_verlauf.dat").lower();

		QString temp="Vor dem Beenden dieses Programmes mssen die Daten IMMER "
			         "manuell per 'Ende'-Taste abge-SPEICHER-t werden.\n\nFast wren "
					 "die wertvollen %1 Sinneinheiten verloren gewesen, die in "
					 "mhseliger %2 dauernden Arbeitszeit eingegeben wurden.\n"
					 "Also: Bitte immer als letztes 'Ende' drcken, dann erst das "
					 "Programm schliessen!\n\nIch werde nun versuchen, eine "
					 "Sicherheitskopie der Daten abzuspeichern unter: \n%3\n%4\n\n"
					 "Nach Kopieren und manuellem Umbenennen ins normale Schema "
					 "knnen die Daten dann einfach\nmittels "
					 "'angefangene Sitzung weiterfhren' eingelesen und neu "
					 "abgespeichert werden.\n\n"
					 "Vielleicht nochmal Glck gehabt? Schaun wir mal, ob das "
					 "Speichern nun klappt ...";
		temp=temp.arg(verloren)
			     .arg(arbeitszeit.toQString())
				 .arg(filename1).arg(filename2);
		QMessageBox::critical( this, "LFTVideo - so beenden ist VERBOTEN!", 
			                      temp);

		VerlaufSpeichern *Speicherer=new VerlaufSpeichern (this);
		 Speicherer->setzePointerAufDaten(Verlauf);
		 QString filenameVerlauf=Speicherer->gibFilename(dasDatum, dieZeit);
		 bool erfolgreichVerlauf = Speicherer->
		                           speichereDaten(filenameVerlauf,"Rating Verlauf");
		delete Speicherer;
		qDebug("Verlauf Speichern war erfolgreich: %d",erfolgreichVerlauf);

		InfoSpeichern *InfoSpeicherer=new InfoSpeichern (this);
		 InfoSpeicherer->merkDirDaten(TextLabelAnzahl->text(), 
								 TextLabelMittlereDauer->text(),
								 Verlauf->BisherLetzterMoment(),
								 weitergefuehrteSession,
								 PauseGedrueckt);
		 QString filenameInfo= InfoSpeicherer->gibFilename(dasDatum, dieZeit);
	 	 bool erfolgreichInfo;
		 erfolgreichInfo = InfoSpeicherer->
		                    speichereDaten(filenameInfo,"Rating Verlauf");
		delete InfoSpeicherer;
		qDebug("InfoSpeichern war erfolgreich: %d",erfolgreichInfo);
	}
	
	// zerstre Daten:
	Verlauf->Sinneinheiten.clear();  delete Verlauf;
	delete KonfigDialog, CountdownTimer, SekundenTimer;

    // no need to delete child widgets, Qt does it all for us
}
