
// zeitverlauf.h
// LFTVideo.cpp  v1.7
//               12.5.2003
// 
// fr weitere Informationen:
// siehe header von main.cpp

#ifndef ZEITVERLAUF_H
#define ZEITVERLAUF_H

// kein Meckern wg. langer Funktionsnamen (wg. map) :
#pragma warning(disable:4786) 

#include <functional>	// fr binary_function "less"
#include <map>			// als Container
#include <iostream>		// fr ostream <<
#include <qstring.h>	// QString
#include <qtextstream.h>// fr Lese und Schreiboperationen auf Platte

using std::binary_function;	
using std::less;
using std::map;
using std::ostream;


// enum RatingSkala
///////////////////

enum RatingSkala {error=-1, null, TL, TNL, TU, KL, KNL, KU, NIX};
const QString RatingSkalaKurz[8]={"null", "TL", "TNL", "TU", "KL", "KNL", "KU", "NIX"};
const QString RatingSkalaLang[8]={"null", 
								  "Therapeut lsungsfokussiert", 
							      "Therapeut nicht lsungsfokussiert", 
								  "Therapeut unklar",
								  "Klient Lsungssprache",
								  "Klient keine Lsungssprache",
								  "Klient unklar",
								  "Pause"};
const int RatingSkalaMax=7;

// helpers
QString leadingZero(int n);
QString leading2Zeros(int n);
double nachkomma (double zahl);
double runden (double zahl);


// class MinSec
///////////////
// Speichert (Minuten, Sekunden)
// incl Vergleichen, Umrechnungen, ...

class MinSec {
private:
	int Min;
	double Sec;
public:
	MinSec(int mins=0, double secs=0, long milliseconds=0);
	int mins() const {return Min;};
	double secs() const {return Sec;};
	int millisecs() const {return runden( 1000 * nachkomma(secs()) );};
	double toSecs() const;				// konvertiere komplett in Sekunden
	double secsTo(const MinSec &rhs) const;  // Abstand in Sekunden
	MinSec addSecs (double s) const;   // liefert MinSec mit s Sekunden spter

	friend bool operator <(const MinSec& x, const MinSec& y);
	friend bool operator ==(const MinSec& x, const MinSec& y);
	friend MinSec operator-(const MinSec& x, const MinSec& y);
	// alt: friend double operator -(const MinSec& x, const MinSec& y);
	struct less : public binary_function<MinSec, MinSec, bool> {};
	
	bool liesVonTextStream(QTextStream &s);
	QString toQString() const;
	static void testFloorInt();
	static void testMinSec() ;
	
};


// struct IntervallErgebnis
///////////////////////////
// Container fr (Rating, RatingDavor, Dauer)

struct IntervallErgebnis {
	RatingSkala Einschaetzung;
	RatingSkala EinschaetzungDavor;
	double Dauer;

	IntervallErgebnis 
		(RatingSkala a=null, RatingSkala b=null, double c=0)
		: Einschaetzung (a),  EinschaetzungDavor (b), Dauer(c) {};

	QString toQString() const;
	friend ostream& operator<< (ostream& os, const IntervallErgebnis& rhs);
};


// map MinSec2Rating
////////////////////
// Container fr (Endzeitpunkt, Rating)

typedef map < MinSec, RatingSkala, MinSec::less >  MinSec2Rating;
void testMinSecMap();


// class RatingVerlauf
//////////////////////
// Container und Memberfunktionen

class RatingVerlauf {
public:
	RatingVerlauf();
	~RatingVerlauf();

	MinSec2Rating Sinneinheiten;

public:
	void testRatingVerlauf();
	// Schreiboperationen
	void RatingLoeschen(const MinSec &Endmoment);
	void RatingAnhaengen(const MinSec &Endmoment, 
						 const RatingSkala Einschaetzung);
	// Leseoperationen:
	MinSec BisherLetzterMoment();
	// liefere alle 3 Daten auf einmal als struct:
	IntervallErgebnis RatingAuslesen (const MinSec &Endmoment);
	IntervallErgebnis RatingAuslesen (MinSec2Rating::iterator it);
	// print Ausgabeoperationen:
	QString printRating (const MinSec &Endmoment);
	QString printGesamtenRatingVerlauf (const MinSec& von, const MinSec& bis);
	bool liesGesamtenRatingVerlauf (QTextStream &s);
	bool speichereGesamtenRatingVerlauf (QTextStream &s, 
		                                QString &derKlient, QString &derRater);
	// bool speichereGesamtenRatingVerlauf(QString filename);

	int verkuerzeUmAufeinanderfolgendGleiche();
};

//struct AnzahlDauer {int Anzahl, QTime Dauer}
	//	Summation (const EinschaetzungsSkala &Einschaetzung);

#endif ZEITVERLAUF_H
